'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var addPlugin = require('@graphql-codegen/add');
var typedDocumentNodePlugin = require('@graphql-codegen/typed-document-node');
var typescriptOperationPlugin = require('@graphql-codegen/typescript-operations');
var typescriptPlugin = require('@graphql-codegen/typescript');
var crypto = require('crypto');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () {
                        return e[k];
                    }
                });
            }
        });
    }
    n['default'] = e;
    return Object.freeze(n);
}

var addPlugin__namespace = /*#__PURE__*/_interopNamespace(addPlugin);
var typedDocumentNodePlugin__namespace = /*#__PURE__*/_interopNamespace(typedDocumentNodePlugin);
var typescriptOperationPlugin__namespace = /*#__PURE__*/_interopNamespace(typescriptOperationPlugin);
var typescriptPlugin__namespace = /*#__PURE__*/_interopNamespace(typescriptPlugin);
var crypto__default = /*#__PURE__*/_interopDefaultLegacy(crypto);

const plugin = (schema, sources, { sourcesWithOperations }) => [
    `import * as graphql from './graphql';\n`,
    `\n`,
    ...getDocumentRegistryChunk(sourcesWithOperations),
    `\n`,
    ...getGqlOverloadChunk(sourcesWithOperations),
    `\n`,
    `export function gql(source: string): unknown;\n`,
    `export function gql(source: string) {\n`,
    `  return (documents as any)[source] ?? {};\n`,
    `}\n`,
].join(``);
function getDocumentRegistryChunk(sourcesWithOperations) {
    const lines = [];
    lines.push(`const documents = {\n`);
    for (const { source: { rawSDL }, operations } of sourcesWithOperations) {
        lines.push(`  ${JSON.stringify(rawSDL)}: {\n`);
        for (const { initialName, uniqueName } of operations)
            lines.push(`    ${initialName}: graphql.${uniqueName}Document,\n`);
        lines.push(`  },\n`);
    }
    lines.push(`};\n`);
    return lines;
}
function getGqlOverloadChunk(sourcesWithOperations) {
    const lines = [];
    // We intentionally don't use a <T extends keyof typeof documents> generic, because TS
    // would print very long `gql` function signatures (duplicating the source).
    for (const { source: { rawSDL } } of sourcesWithOperations)
        lines.push(`export function gql(source: ${JSON.stringify(rawSDL)}): (typeof documents)[${JSON.stringify(rawSDL)}];\n`);
    return lines;
}

var dtsGenPlugin = /*#__PURE__*/Object.freeze({
    __proto__: null,
    plugin: plugin
});

function processSources(sources) {
    var _a, _b;
    const fullHashes = new Map();
    for (const source of sources) {
        if (!source)
            continue;
        const hash = crypto__default['default'].createHash(`sha256`).update(source.rawSDL).digest(`hex`);
        if (fullHashes.has(hash))
            continue;
        fullHashes.set(hash, source);
    }
    let hashLength = 0;
    findHashSize: for (let t = 1; t <= 32; ++t) {
        const seen = new Set();
        for (const hash of fullHashes.keys()) {
            const sub = hash.substr(0, t);
            if (seen.has(sub)) {
                continue findHashSize;
            }
            else {
                seen.add(sub);
            }
        }
        hashLength = t;
        break;
    }
    const subHashes = new Map();
    for (const [hash, record] of fullHashes)
        subHashes.set(hash.substr(0, hashLength), record);
    const sourcesWithOperations = [];
    for (const [hash, source] of subHashes) {
        const { document } = source;
        const operations = [];
        for (const definition of (_a = document === null || document === void 0 ? void 0 : document.definitions) !== null && _a !== void 0 ? _a : []) {
            if ((definition === null || definition === void 0 ? void 0 : definition.kind) !== `OperationDefinition`)
                continue;
            if (((_b = definition.name) === null || _b === void 0 ? void 0 : _b.kind) !== `Name`)
                continue;
            // The leading 'X' in front of the hash is to prevent graphql-code-generator
            // from accidentally uppercasing the first letter of the hash.
            const initialName = definition.name.value;
            const uniqueName = `${initialName}_X${hash}_`;
            Object.assign(definition.name, { value: uniqueName });
            operations.push({
                initialName,
                uniqueName,
                definition,
            });
        }
        if (operations.length === 0)
            continue;
        sourcesWithOperations.push({
            source,
            operations,
        });
    }
    return sourcesWithOperations;
}

const preset = {
    buildGeneratesSection: options => {
        var _a;
        const packageName = (_a = options.presetConfig.packageName) !== null && _a !== void 0 ? _a : `@app/gql`;
        const sourcesWithOperations = processSources(options.documents);
        const sources = sourcesWithOperations.map(({ source }) => source);
        const pluginMap = Object.assign(Object.assign({}, options.pluginMap), { [`add`]: addPlugin__namespace, [`typescript`]: typescriptPlugin__namespace, [`typescript-operations`]: typescriptOperationPlugin__namespace, [`typed-document-node`]: typedDocumentNodePlugin__namespace, [`gen-dts`]: dtsGenPlugin });
        const plugins = [
            { [`add`]: { content: `/* eslint-disable */` } },
            { [`typescript`]: {} },
            { [`typescript-operations`]: {} },
            { [`typed-document-node`]: {} },
            ...options.plugins,
        ];
        const genDtsPlugins = [
            { [`add`]: { content: `/* eslint-disable */` } },
            { [`gen-dts`]: { sourcesWithOperations, packageName } },
        ];
        return [{
                filename: `${options.baseOutputDir}/graphql.ts`,
                plugins,
                pluginMap,
                schema: options.schema,
                config: options.config,
                documents: sources,
            }, {
                filename: `${options.baseOutputDir}/index.ts`,
                plugins: genDtsPlugins,
                pluginMap,
                schema: options.schema,
                config: options.config,
                documents: sources,
            }];
    },
};

exports.preset = preset;
